﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Runtime.CompilerServices;

namespace IDataErrorInfo_ErrorTooltip
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = new MainViewModel();
        }
    }

    public class MainViewModel : ViewModelVase
    {
        private ObservableCollection<Data> data;

        public MainViewModel()
        {
            DataSource = new ObservableCollection<Data>();

            for (int i = 0; i < 10; i++)
            {
                DataSource.Add(new Data
                {
                    Name = "Item " + i,
                    Size = new Random().Next(20) + new Random().NextDouble(), 
                });

                
            }
        }

        public ObservableCollection<Data> DataSource
        {
            get { return data; }
            set { data = value; }
        }

    }

    public class Data : ViewModelVase
    {
        private string name;
        private double size;

        public string Name
        {
            get { return name; }
            set
            {
                IsNameValid(value);
                name = value;
                OnPropertyChagned("Name");
            }

        }

        public double Size
        {
            get { return size; }
            set
            {
                IsSizeValid(value);

                size = value;
                OnPropertyChagned("Size");
            }
        }



        #region Validate
        private bool IsSizeValid(double value)
        {
            if (value < 0)
            {
                AddError("Size", "Size must be non-negative");
                return false;
            }
            else
            {
                RemoveError("Size", "Size must be non-negative");
                return true;
            }
        }
        private bool IsNameValid(string name)
        {
            if (string.IsNullOrEmpty(name) || name.Length < 3)
            {
                AddError("Name", "Name must be 3 chars or more");
                return false;
            }
            else
            {
                RemoveError("Name", "Name must be 3 chars or more");                 
                return true;
            }
        }
        #endregion


    }
    
    public class ViewModelVase : INotifyPropertyChanged, IDataErrorInfo
    {

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion


        #region IDataErrorInfo members
        public string rowError = "";
        Dictionary<string, List<string>> errors = new Dictionary<string, List<string>>();

        public string Error
        {
            get { return rowError; }
        }

        [IndexerName("Item")]
        public string this[string columnName]
        {
            get
            {
                return (!errors.ContainsKey(columnName) ? null :
                      String.Join(Environment.NewLine, errors[columnName]));
            }
        }

        //Add errors
        public void AddError(string prop, string err)
        {
            if (!errors.ContainsKey(prop))
                errors.Add(prop, new List<string>());
            if (!errors[prop].Contains(err))
                errors[prop].Add(err);
            OnPropertyChagned("Item[]");

        }
        //Remove errors
        public void RemoveError(string prop, string err)
        {
            if (errors.ContainsKey(prop) &&
              errors[prop].Contains(err))
            {
                errors[prop].Remove(err);
                if (errors[prop].Count == 0) errors.Remove(prop);
            }
        }
        #endregion
    }
}
